#!/bin/csh
#
# Author: R.A. Jansen -- Mar 30, 2003
#
# shell script to monitor the status of running programs.  If more than the
# specified number of instances of the same program are running simultaneously,
# or if an instance of the specified program is running longer than a specified
# time, assume the top program in the list is hanging: intervene and kill it.

if ( "" == "$1" || "-h" == "$1" || "--help" == "$1" ) then
    echo "usage: jobmon.csh -p <prog_name> [-k #] [-m <sec>] [-d <sec>] [-v]"
    echo 'progname  is a minimal unique match to the name of the program;'
    echo '-k #      specifies the maximum number "#" of instances of the speci-'
    echo '          fied program that may run simultaneously [default 1];'
    echo '-m <sec>  specifies the monitoring frequency in seconds [default 10]'
    echo '-d <sec>  specifies the maximum duration in seconds that an instance'
    echo '          of the specified program may run [default 900]'
    echo '-v        verbose: normally a message is printed only when a process'
    echo '          is killed.'
    exit
endif

# defaults
set prgs = "a.out"
set kmax = 1
set usec = 10
set dsec = 900
set loud = 0

while ( a == a )
    if ( "-p" == "$1" ) then
	set prgs = "$2" ; shift ; shift
    else if ( "-k" == "$1" ) then
	set kmax = $2 ; shift ; shift
    else if ( "-m" == "$1" ) then
	set usec = $2 ; shift ; shift
    else if ( "-d" == "$1" ) then
	set dsec = $2 ; shift ; shift
    else if ( "-v" == "$1" || "--verbose" == "$1" ) then
	set loud = 1 ; shift
    else
	break
    endif
end
                
set nd=`echo "$dsec $usec" | awk '{print int($1/$2) }'`
                
echo 'Monitoring "'$prgs'" processes (nmax='$kmax', dmax='$dsec' sec) every '$usec' sec'
echo "until receiving ^C ..."
        
set oprg = 0
set np = 0    
while ( a == a )
    #set nprg = `ps -a | grep "$prgs" | grep -v 'grep' | wc -l`
    #set pprg = `ps -a | grep "$prgs" | grep -v 'grep' | awk '{print $1}' | head -1`
    set nprg = `ps -C "$prgs" -o ruser=RealUser,pid | grep hst | wc -l`
    set pprg = `ps -C "$prgs" -o ruser=RealUser,pid | grep hst | awk '{print $2}' | head -1`

    if ( 0 < $nprg && $pprg == $oprg ) then
	@ np = $np + 1
    else
	set np = 0
    endif
    set nsec=`echo "$np $usec" | awk '{print int($1*$2) }'`
    if ( 0 < $loud ) then
	echo `date +'%D %T'`': nprg = '$nprg', pprg = '$pprg', nsec = '$nsec
    endif
    if ( $kmax < $nprg || $nd < $np ) then
	if ( 0 == $loud ) then
	    echo `date +'%D %T'`': nprg = '$nprg', pprg = '$pprg', nsec = '$nsec
	endif
	if ( "idl" == "$prgs" ) then
	   grep 'PSUB' tmpdoidl
	endif
	( kill -9 $pprg ) &
	echo `date`": killed process "$pprg
	set np = 0
    endif
    set oprg = $pprg
    sleep $usec
end

