C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      SUBROUTINE error(csub,cstr)

C @(#)Subroutine: error              Author: R.A. Jansen -- Oct 24, 2000
C @(#)
C @(#)  Purpose: Write an error message  'cstr'  to STDOUT, the standard
C @(#)    output device. 'csub' may be used to pass a string identifying
C @(#)    the calling routine.  If necessary,  the error message will be
C @(#)    automatically broken over multiple 80-character lines.
C @(#)
C @(#)  Calling sequence: CALL error(csub,cstr)  where 'cstr' and 'csub'
C @(#)    are character strings of arbitrary declared length.
C @(#)
C @(#)  Dependencies:     clen, cinit; 'devices.inc'
C @(#)
C @(#)  Revision history:
C @(#)    May 24, 2008 [RAJ] -- expanded documentation.
C @(#)    Jun 10, 2008 [RAJ] -- re-implemented to allow breaking message
C @(#)                          of arbitrary length over  multiple lines
C @(#)                          at word boundaries.
C @(#)    Aug  3, 2008 [RAJ] -- fixed buggy code for multi-line message.
C @(#)------------------------------------------------------------------

      IMPLICIT NONE

C --- Learn about default devices...
      INCLUDE	'devices.inc'

C --- Declaration of input variables...
      CHARACTER*(*) csub, cstr

C --- Declaration of local variables...
      CHARACTER*80 errmsg, ctmp
      INTEGER	clen, i, j, k, m

C --- Initialize...
      CALL cinit(errmsg)
      i = clen(csub)
      j = clen(cstr)
      m = 0

C --- Initialize the error message...
      IF (i.GT.0) THEN
         errmsg = 'ERROR('//csub(1:i)//'): '
      ELSE
         errmsg = 'ERROR: '
      ENDIF
      i = clen(errmsg) + 1
      IF (j.EQ.0) THEN
C        'cstr' was blank; issue a generic error message...
         WRITE(STDOUT,'(A)') errmsg(1:i)//'somewhere'
         RETURN
      ENDIF

C --- Construct and print the error message...
      DO WHILE (j.GT.0)
         IF (j.LE.(80-i)) THEN
C           We can print the (remainder of the) message at once...
            k = j
         ELSE
C           We'll have to break the (remainder of the) message over
C           multiple lines; only break a line at the end of a word...
            k = 80 - i
            ctmp = cstr((m+1):(m+k))

C           Find the last blank space from the end of the current line..
            DO WHILE (ctmp(k:k).NE.CHAR(32))
               k = k - 1
            ENDDO
         ENDIF
         errmsg = errmsg(1:i)//cstr((m+1):(m+k))
         WRITE(STDOUT,'(A)') errmsg(1:clen(errmsg))

C        Precede all continuation lines with 6 blank characters...
         errmsg = '      '
         i = 6

         j = j - k
         m = m + k
      ENDDO

      RETURN
      END


