C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++   

      SUBROUTINE rjf_close (iunit, idbg, ierr)

C @(#)Subroutine: rjf_close          Author: R.A. Jansen -- May 28, 2009
C @(#)
C @(#)  Purpose:  Close a file currently open at specific logical device
C @(#)    unit 'iunit'. If 'idbg'>0, additional messages may be printed.
C @(#)    Integer error status 'ierr' will be non-zero upon exit, should
C @(#)    any error condition be detected. Its value (if >2000) uniquely
C @(#)    identifies a known error condition.
C @(#)
C @(#)  Calling sequence:  CALL rjf_close (iunit, idbg, ierr), where in-
C @(#)    teger  'iunit'  is a logical device unit  (possibly a standard
C @(#)    device) and 'idbg' and 'ierr' are both integers.
C @(#)
C @(#)  Dependencies:	clen; 'devices.inc'
C @(#)
C @(#)  Revision history:
C @(#)    Jul  1, 2009 [RAJ] -- Fixed bug of possibly negative 'iunit'.
C @(#)------------------------------------------------------------------

      IMPLICIT NONE

C --- Learn about default devices...
      INCLUDE	'devices.inc'

C --- Declaration of input/return variables...
      INTEGER	iunit, idbg, ierr

C --- Declaration of local variables...
      CHARACTER	dbid*19, fname*80
      INTEGER	clen
      LOGICAL	lopen

C     ------------------------------------------------------------------

C --- Return for non-zero input error status...
      IF (ierr.NE.0) RETURN

C     Initialize the debugging ID string...
      dbid  = 'DEBUG(rjf_close): '

C     Return if 'iunit' is negative or matches a reserved device...
      IF (iunit.LT.0) THEN
         IF (idbg.GT.0) WRITE(*,*) dbid//'specified logical device '//
     +      'unit number is illegal (',iunit,'< 0).'
         ierr = 2011
      ELSE IF (iunit.EQ.STDIN) THEN
         IF (idbg.GT.0) WRITE(*,*) dbid//
     +      'specified logical device unit number matches STDIN.'
         ierr = 2012
      ELSE IF (iunit.EQ.STDOUT) THEN
         IF (idbg.GT.0) WRITE(*,*) dbid//
     +      'specified logical device unit number matches STDOUT.'
         ierr = 2013
      ELSE IF (iunit.EQ.STDERR) THEN
         IF (idbg.GT.0) WRITE(*,*) dbid//
     +      'specified logical device unit number matches STDERR.'
         ierr = 2014
      ENDIF
      IF (ierr.NE.0) RETURN

C --- Test that a file is indeed attached to the specified logical
C     device unit and open...
      lopen = .FALSE.
      INQUIRE (UNIT=iunit,OPENED=lopen,NAME=fname)
      IF (.NOT.lopen) THEN
         IF (idbg.GT.0) WRITE(*,*) dbid//'specified logical device '//
     +      'unit number is not attached to an open file'
         ierr = 2015
         RETURN
      ENDIF

C --- Close the file attached to the specified logical device unit...
      CLOSE (UNIT=iunit,STATUS='KEEP',IOSTAT=ierr)
      IF (ierr.NE.0) THEN
         IF (idbg.GT.0) WRITE(*,*) dbid//'error closing file '//
     +      fname(1:clen(fname))//' attached to logical device unit ',
     +      iunit
         ierr = 2016
      ENDIF


      RETURN
      END


